package ms.test;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;

import org.apache.log4j.Logger;
import org.junit.Assert;
import org.junit.Test;
import org.modeshape.jcr.ModeShapeEngine;
import org.modeshape.jcr.RepositoryConfiguration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;

public class MixinIndexTest {

    private static final Logger LOG = Logger.getLogger(MixinIndexTest.class);

    private static final String REPO_CONFIG_LOCATION = "/mixinIndex/repo-config.json";

    @Test
    public void testMixinIndex() throws RepositoryException {
        final ModeShapeEngine engine = new ModeShapeEngine();
        engine.start();
        final RepositoryConfiguration configuration = getRepositoryConfiguration(REPO_CONFIG_LOCATION);
        final Repository repository = engine.deploy(configuration);
        final Session session = repository.login("default");

        final Node root = session.getRootNode();
        final Node node0 = root.addNode("node0");
        node0.setPrimaryType("my:content");
        node0.setProperty("content", "test content 0");
        node0.setProperty("author", "author1");
        node0.setProperty("coAuthors", new String[] { "author2", "author3" });

        final Node node1 = root.addNode("node1");
        node1.setPrimaryType("my:content");
        node1.setProperty("content", "test content 1");
        node1.setProperty("author", "author2");

        session.save();

        final List<Node> queryResults = executeQuery(session, "author1"); // should get node0 (once), but gets node0 twice
        Assert.assertEquals(1, queryResults.size());

        //executeQuery(session, "author2"); // should get node0 and node1, but gets node1 twice
    }

    private List<Node> executeQuery(final Session session, final String author) throws RepositoryException {
        final List<Node> results = new ArrayList<Node>();
        final StringBuilder sql = new StringBuilder();
        sql.append("select * from [my:content] ");
        sql.append(" where author in ($author) ");
        sql.append(" or coAuthors in ($author) ");

        final QueryManager queryManager = session.getWorkspace().getQueryManager();
        final Query query = queryManager.createQuery(sql.toString(), Query.JCR_SQL2);
        query.bindValue("author", session.getValueFactory().createValue(author));
        final QueryResult result = query.execute();
        final NodeIterator nodeIterator = result.getNodes();
        while (nodeIterator.hasNext()) {
            final Node node = (Node) nodeIterator.next();
            LOG.info(node.getIdentifier() + ", " + node.getProperty("content") + ", " + node.getProperty("author") + ", " + (node.hasProperty("coAuthors") ? node.getProperty("coAuthors") : "no coAuthors"));
            results.add(node);
        }
        return results;
    }

    protected RepositoryConfiguration getRepositoryConfiguration(final String configFileName) {
        try {
            final Resource res = new ClassPathResource(configFileName);
            final RepositoryConfiguration config = RepositoryConfiguration.read(res.getURL());
            final org.modeshape.common.collection.Problems problems = config.validate();
            if (problems.hasErrors()) {
                Assert.fail();
            }
            return config;
        } catch (final Throwable e) {
            LOG.error(e.getMessage(), e);
            Assert.fail();
        }
        return null;
    }
}
